import { Actor } from '../display/actor.js';
import { Action } from '../display/action.js';
import { CryptoUtil } from '../util/crypto.js';
import { PrioritizedQueue } from '../util/prioritized-queue.js';

export class NetworkNode extends Actor {
  constructor(name, scene) {
    super(name, scene);
    this.queue = new PrioritizedQueue();
    this.actions = {
      peerMessage: new Action('peer message', scene),
    };
  }

  // Generate a signing key pair and connect to the network
  async initialize(forumNetwork) {
    this.keyPair = await CryptoUtil.generateAsymmetricKey();
    this.forumNetwork = forumNetwork.addNode(this);
    this.status.set('Initialized');
    return this;
  }

  // Send a message to all other nodes in the network
  async broadcast(message) {
    await message.sign(this.keyPair);
    const otherForumNodes = this.forumNetwork
      .listNodes()
      .filter((forumNode) => forumNode.keyPair.publicKey !== this.keyPair.publicKey);
    for (const forumNode of otherForumNodes) {
      // For now just call receiveMessage on the target node
      // await this.actions.peerMessage.log(this, forumNode, null, message.content);
      await this.actions.peerMessage.log(this, forumNode);
      await forumNode.receiveMessage(JSON.stringify(message.toJSON()));
    }
  }

  // Perform minimal processing to ingest a message.
  // Enqueue it for further processing.
  async receiveMessage(messageStr) {
    const messageJson = JSON.parse(messageStr);
    // const senderReputation = this.forumView.getReputation(messageJson.publicKey) || 0;
    const senderReputation = 0;
    this.queue.add(messageJson, senderReputation);
  }

  // Process next highest priority message in the queue
  async processNextMessage() {
    const messageJson = this.queue.pop();
    if (!messageJson) {
      return null;
    }
    return this.processMessage(messageJson);
  }

  // Process a message from the queue
  // async processMessage(messageJson) {
  // }
}
